@@ -1,3 +1,5 @@
+//! Abstract async client.
+
 mod helpers;
 mod manager;
 
@@ -119,6 +121,7 @@ impl ClientBuilder {
 	///
 	/// Panics if called outside of `tokio` runtime context.
 	#[cfg(feature = "async-client")]
+	#[cfg_attr(docsrs, doc(cfg(feature = "async-client")))]
 	pub fn build_with_tokio<S, R>(self, sender: S, receiver: R) -> Client
 	where
 		S: TransportSenderT + Send,
@@ -141,6 +144,7 @@ impl ClientBuilder {
 
 	/// Build the client with given transport.
 	#[cfg(all(feature = "async-wasm-client", target_arch = "wasm32"))]
+	#[cfg_attr(docsrs, doc(cfg(feature = "async-wasm-client")))]
 	pub fn build_with_wasm<S, R>(self, sender: S, receiver: R) -> Client
 	where
 		S: TransportSenderT,
